/*********************************************
Chip type           : AT90S8535
Clock frequency     : 8,000000 MHz
Memory model        : Small
Internal SRAM size  : 512
External SRAM size  : 0
Data Stack size     : 128
*********************************************/

#include <90s8535.h>
#include <delay.h>
#include <stdio.h> 

// Alphanumeric LCD Module functions
#asm
   .equ __lcd_port=0x15
#endasm
#include <lcd.h>  

#define US_TXD        PORTD.6       

#define TASTE1 PINA.7
#define TASTE2 PINA.6
#define TASTE3 PINA.5
#define TASTE4 PINA.4
#define TASTE5 PINA.3



#define KORREKTUR     17              // Correction (in 0,5cm)
#define DISTANCE_MIN  30              // min.distance (in cm) 
                               

unsigned int Distance[5];        // 5 Entfernungen in einer Richtung
static unsigned char Weiter,index =0;  // Index fr Entfernungsarray
unsigned int  Ausblenden = 0;                 
char text[16]; 

void US_Messung(void)
{
         GIMSK &=~0x80;
         TCNT1 = 0;                      // Timer reset
         TCCR1B=0x04;                    // Timer start 
         Weiter = 0;     

         US_TXD = 0; 
         Distance[index] = 0;
         delay_us(50);
         US_TXD = 1;
         delay_us(2500);
      
         GIFR  |= 0x80;
         GIMSK |=0x80;

}


// External Interrupt 1 service routine
interrupt [EXT_INT1] void ext_int1_isr(void)
{  
 // Place your code here
//if(!Distance) 
  if (TCNT1 > Ausblenden)        // 
  {
    if (((TCNT1/2) - KORREKTUR) > DISTANCE_MIN)
    { 
      Distance[index] = (TCNT1/2) - KORREKTUR; 
      if (Distance[index]>9999) { Distance[index]=9999; index = 3; }
      Ausblenden = TCNT1 + 60;
      index++;
    }
     else Ausblenden = TCNT1 + 10;   // min. distance between 2 object       
     
    if (index == 4 ) 
    {
      GIMSK &= ~0x80;     
      TCCR1B = 0x00;       // Timer stop
      index = 0;
      Ausblenden = 0;
    }
  }  
}

// Standard Input/Output functions
#include <stdio.h>

// Timer 1 overflow interrupt service routine
interrupt [TIM1_OVF] void timer1_ovf_isr(void)
{
// Place your code her
    GIMSK &= ~0x80;     
    TCCR1B = 0x00;       // Timer stop
    Weiter = 1;  
    Distance[index] = 9999;
    index = 0;
    Ausblenden = 0;    
}

                   

//--------------------------------------------------------------------
void main(void)
{
// Declare your local variables here

// Input/Output Ports initialization
// Port A
delay_ms(250);
PORTA=0x00;
DDRA=0x00;
                   

// Port B
PORTB=0x00;
DDRB=0x0F;

// Port C
PORTC=0x00;
DDRC=0x00;

// Port D
PORTD=0x60;    // Pullup fr Endschalter
DDRD=0x40;

// Timer/Counter 0 initialization
// Clock source: System Clock
// Clock value: Timer 0 Stopped
// Mode: Output Compare
// OC0 output: Disconnected
TCCR0=0x00;
TCNT0=0x00;

// Timer/Counter 1 initialization
// Clock source: System Clock
// Clock value: 31,250 kHz
// Mode: Output Compare
// OC1A output: Discon.
// OC1B output: Discon.
// Noise Canceler: Off
// Input Capture on Falling Edge
TCCR1A=0x00;
TCCR1B=0x04;
TCNT1H=0x00;
TCNT1L=0x00;
OCR1AH=0x00;
OCR1AL=0x00;
OCR1BH=0x00;
OCR1BL=0x00;

// Timer/Counter 2 initialization
// Clock source: System Clock
// Clock value: Timer 2 Stopped
// Mode: Output Compare
// OC2 output: Disconnected
TCCR2=0x00;
ASSR=0x00;
TCNT2=0x00;
OCR2=0x00;

// External Interrupt(s) initialization
// INT0: Off
// INT1: On
// INT1 Mode: Falling Edge
GIMSK=0x80;
MCUCR=0x08;
GIFR=0x80;

// Timer(s)/Counter(s) Interrupt(s) initialization
TIMSK=0x04;

// UART initialization
// Communication Parameters: 8 Data, 1 Stop, No Parity
// UART Receiver: On
// UART Transmitter: On
// UART Baud rate: 9600 
UCR=0x98;
UBRR=0x19;
//UCR=0x18;
//UBRR=0x33;

// Analog Comparator initialization
// Analog Comparator: Off
// Analog Comparator Input Capture by Timer/Counter 1: Off
ACSR=0x80;

// LCD module initialization
lcd_init(16);

// Global enable interrupts
#asm("sei")             
             
                        
index = 0;              
  

while (1)
      {                 
         unsigned char i;
         
         Distance[index] = 0;  
         delay_ms(750);
         US_Messung();                 // 
         
        while(!Distance[index] && !Weiter); 
      
         lcd_gotoxy(0,0);
         i=0;
         while (i<4)
         {
           sprintf(text,"%3d",Distance[i++]);
           lcd_puts(text);
         }                            
         
      }
}      